from microbit import display, sleep, button_a
import audio
import math

def repeated_frame(frame, count):
    for i in range(count):
        yield frame

# Press button A to skip to next wave.
def show_wave(name, frame, duration=1500):
    display.scroll(name + " wave", wait=False,delay=100)
    audio.play(repeated_frame(frame, duration),wait=False)
    for i in range(75):
        sleep(100)
        if button_a.is_pressed():
            display.clear()
            audio.stop()
            break

frame = audio.AudioFrame()

for i in range(len(frame)):
    frame[i] = int(math.sin(math.pi*i/16)*124+128.5)
show_wave("Sine", frame)

triangle = audio.AudioFrame()

QUARTER = len(triangle)//4
for i in range(QUARTER):
    triangle[i] = i*15
    triangle[i+QUARTER] = 248-i*15
    triangle[i+QUARTER*2] = 128-i*15
    triangle[i+QUARTER*3] = i*15+8
show_wave("Triangle", triangle)

square = audio.AudioFrame()

HALF = len(square)//2
for i in range(HALF):
    square[i] = 8
    square[i+HALF] = 248
show_wave("Square", square)
sleep(1000)

for i in range(len(frame)):
    frame[i] = 252-i*8
show_wave("Sawtooth", frame)

del frame

#Generate a waveform that goes from triangle to square wave, reasonably smoothly.
frames = [ None ] * 32
for i in range(32):
    frames[i] = frame = audio.AudioFrame()
    for j in range(len(triangle)):
        frame[j] = (triangle[j]*(32-i) + square[j]*i)>>5

def repeated_frames(frames, count):
    for frame in frames:
        for i in range(count):
            yield frame


display.scroll("Ascending wave", wait=False)
audio.play(repeated_frames(frames, 60))



"""
Dodge game

Get the player back and forth across the screen while dodging the enemy
"""

from microbit import *
import music


class Enemy:
    """
    Enemy which moves vertically down the screen
    """
    def __init__(self):
        self.x, self.y = 2, -1

    def get_positions(self):
        return ((self.x, self.y), (self.x, self.y + 1 if self.y < 4 else 0))

    def move(self):
        # Rotate back round to the top
        self.y = (self.y + 1) % 5

    def draw(self):
        for x, y in self.get_positions():
            display.set_pixel(x, y, 9)


class Player:
    """
    Left-right moving player which can be controlled with buttons
    """
    RIGHT = 1
    LEFT = -1
    STOPPED = 0
    LEFT_EDGE = 0
    RIGHT_EDGE = 4

    def __init__(self):
        self.alive = True
        self.score = 0
        self.just_scored = False
        self.x, self.y = self.LEFT_EDGE, 2
        self.direction = self.STOPPED

    def get_position(self):
        return (self.x, self.y)

    def die(self):
        """
        Player dies - show their score and play sad music
        """
        self.alive = False
        display.show(str(self.score))
        music.play(music.WAWAWAWAA)

    def move(self):
        """
        Move the player one step further in their
        current direction
        """
        self.just_scored = False
        self.x += self.direction
        if self.x in (self.LEFT_EDGE, self.RIGHT_EDGE):
            # Player reached the edge - another run survived!
            if self.direction != self.STOPPED:
                self.score += 1
                self.just_scored = True

            self.direction = self.STOPPED

    def draw(self):
        """
        Draw the player
        """
        display.set_pixel(self.x, self.y, 9)
        if self.just_scored:
            music.pitch(400, 40)

    def act_on_input(self):
        # If we're standing still, look for a button press.
        if self.direction == self.STOPPED:
            if button_b.was_pressed() and self.x == self.LEFT_EDGE:
                self.direction = self.RIGHT

            elif button_a.was_pressed() and self.x == self.RIGHT_EDGE:
                self.direction = self.LEFT


class Game:
    def __init__(self):
        self.enemy = Enemy()
        self.player = Player()
        self.frame_rate = 1

    def detect_collisions(self):
        """
        Have the player and the enemy collided?
        """
        return self.player.get_position() in self.enemy.get_positions()

    def do_frame(self):
        """
        Called once per frame to advance the game state
        """
        # Adjust the speed as the player's score gets higher
        # (But don't let it exceed the actual frame rate)
        self.frame_rate = max(1, min(100, self.player.score))

        if self.player.alive:
            display.clear()

            self.enemy.move()
            self.player.act_on_input()
            self.player.move()

            if self.detect_collisions():
                self.player.die()
            else:
                self.enemy.draw()
                self.player.draw()


game = Game()
while True:
    timestamp = running_time()

    game.do_frame()

    # Keep the frame rate consistent
    new_timestamp = running_time()
    time_taken = (new_timestamp - timestamp)
    interval = 1000 // game.frame_rate
    if time_taken < interval:
        sleep(interval - time_taken)
    timestamp = new_timestamp



"""
A simple maze program.  You are the flashing dot and can walk around
using the accelerometer.
"""

import microbit

d = microbit.display
ac = microbit.accelerometer

# the maze data, as binary numbers (outside walls are added automatically)
maze = [
    0b0000000000000000,
    0b0100010101011110,
    0b0100010101010010,
    0b0111110100000000,
    0b0000000111111110,
    0b0111111101000000,
    0b0101010001011100,
    0b0101000100000100,
    0b0100011111111100,
    0b0101010001000110,
    0b0101000100010010,
    0b0101010111010110,
    0b0111010101010010,
    0b0000010100010010,
    0b0111110111111110,
    0b0000000000000000,
]

def get_maze(x, y):
    if 0 <= x < 16 and 0 <= y < 16:
        return (maze[y] >> (15 - x)) & 1
    else:
        return 1

def draw(x, y, tick):
    img = microbit.Image(5,5)
    for j in range(5):
        for i in range(5):
            img.set_pixel(i, j, get_maze(x + i - 2, y + j - 2)*5)

    # draw the player, flashing
    img.set_pixel(2, 2, (tick & 1)*4+5)
    d.show(img)

def main():
    x = 0
    y = 0
    tick = 0
    while True:
        tick += 1
        if tick == 4:
            # walk around, with collision detection
            tick = 0
            if ac.get_x() > 200 and get_maze(x + 1, y) == 0:
                x += 1
            elif ac.get_x() < -200 and get_maze(x - 1, y) == 0:
                x -= 1
            elif ac.get_y() > 200 and get_maze(x, y + 1) == 0:
                y += 1
            elif ac.get_y() < -200 and get_maze(x, y - 1) == 0:
                y -= 1
            x = min(15, max(0, x))
            y = min(15, max(0, y))

        # draw the maze
        draw(x, y, tick)


        microbit.sleep(50)

main()



#This tests that the duty cycle of PWM is (approximately) correct.

from microbit import *

def pins_connected(p0, p1):
    for i in 0,1,0,1,0,1,0,1:
        p0.write_digital(i)
        if p1.read_digital() != i:
            return False
        p1.write_digital(i)
        if p0.read_digital() != i:
            return False
    return True

def test_pwm(p0, p1):
    #Flip between modes many times
    for i in range(100):
        pin0.write_digital(i & 1)
        pin0.write_analog(i*10)
    #Now make sure it is still working.
    p0.write_digital(0)
    for val in 100, 200, 300, 500, 800, 900:
        print("Testing duty cycle", val)
        p0.write_analog(val)
        sleep(20)
        count = 0
        for i in range(10*1024):
            count += p1.read_digital()
        print("%+0.2f%%" % ((count-val*10)/val*10))
        # Allow +- 5%
        assert abs(count - val*10) < val//2

try:
    if pins_connected(pin0, pin1):
        test_pwm(pin0, pin1)
        print("PWM test: PASS")
        display.show(Image.HAPPY)
    else:
        print("Connect pin0 to pin1")
        display.show("?")
except Exception as ae:
    display.show(Image.SAD)
    raise




#!/usr/bin/env python3

'''
Turn a Python script into Intel HEX format to be concatenated at the
end of the MicroPython firmware.hex.  A simple header is added to the
script.

To execute from command line: ./hexlifyscript.py <script.py>
It also accepts data on standard input.
'''

import struct
import binascii
import fileinput


IHEX_TYPE_DATA = 0
IHEX_TYPE_EXT_LIN_ADDR = 4

SCRIPT_ADDR = 0x3e000 # magic start address in flash of script

def make_ihex_record(addr, type, data):
    record = struct.pack('>BHB', len(data), addr & 0xffff, type) + data
    checksum = (-(sum(record))) & 0xff
    return ':%s%02X' % (str(binascii.hexlify(record), 'utf8').upper(), checksum)

def hexlify_script(script):
    # add header, pad to multiple of 16 bytes
    data = b'MP' + struct.pack('<H', len(script)) + script
    data = data + bytes(16 - len(data) % 16)
    assert len(data) <= 0x2000

    # convert to .hex format
    output = []
    addr = SCRIPT_ADDR
    assert(SCRIPT_ADDR >> 16 == 3) # 0x0003 is hard coded in line below
    output.append(make_ihex_record(0, IHEX_TYPE_EXT_LIN_ADDR, b'\x00\x03'))
    for i in range(0, len(data), 16):
        chunk = data[i:min(i + 16, len(data))]
        output.append(make_ihex_record(addr, IHEX_TYPE_DATA, chunk))
        addr += 16

    return '\n'.join(output)

if __name__ == '__main__':
    # read script from a file and print out the hexlified version
    with fileinput.input(mode='rb') as lines:
        print(hexlify_script(b''.join(lines)))




from microbit import display, Image

def eq(i, j):
    w = i.width()
    h = i.height()
    if w != j.width():
        return False
    if h != j.height():
        return False
    for x in range(w):
        for y in range(h):
            if i.get_pixel(x,y) != j.get_pixel(x,y):
                return False
    return True

TEST = Image("44444:45554:45654:45554:44444")

def test_blit():
    assert eq(TEST, TEST)
    i = Image(4, 4)
    i.blit(TEST, 1, 1, 5, 5)
    assert eq(i, Image("5554:5654:5554:4444"))
    i.fill(2)
    i.blit(TEST, -2, -2, 3, 3)
    assert eq(i, Image('0002:0002:0042:2222'))
    i.fill(2)
    i.blit(TEST, 2, 2, 3, 3)
    assert eq(i, Image('6542:5542:4442:2222'))
    i.fill(2)
    i.blit(TEST, 0, 0, 5, 5, -3, -3)
    assert eq(i, Image('5422:4422:2222:2222'))
    i.fill(2)
    i.blit(TEST, 2, 2, 2, 2, 1, 1)
    assert eq(i, Image('2222:2652:2552:2222'))
    i = TEST.copy()
    i.blit(i, 2, 2, 3, 3)
    assert eq(i, Image('65444:55454:44454:45554:44444'))
    i = TEST.copy()
    i.blit(i, -2, -2, 5, 5)
    assert eq(i, Image('00000:00000:00444:00455:00456'))
    i = TEST.copy()
    i.blit(i, 0, 0, 3, 3, 2, 2)
    assert eq(i, Image('44444:45554:45444:45455:44456'))
    i = Image(2, 7)
    i.fill(2)
    i.blit(TEST, -100, -100, 50, 50, 1, 1)
    assert eq(i, Image('22:20:20:20:20:20:20'))

def test_crop():
    assert eq(TEST.crop(-1, -1, 2, 2), Image('00:04'))
    assert eq(TEST.crop(1, 1, 2, 2), Image('55:56'))
    assert eq(TEST.crop(4, 4, 3, 3), Image('400:000:000'))

def test_shift():
    assert eq(TEST, TEST.shift_left(0))
    assert eq(TEST, TEST.shift_up(0))
    for n in range(-6, 7):
        assert eq(TEST.shift_left(n), TEST.shift_right(-n))
        assert eq(TEST.shift_up(n), TEST.shift_down(-n))
    assert eq(TEST.shift_left(1), Image('44440:55540:56540:55540:44440'))
    assert eq(TEST.shift_down(1), Image('00000:44444:45554:45654:45554'))

try:
    display.scroll("blit")
    test_blit()
    display.scroll("crop")
    test_crop()
    display.scroll("shift")
    test_shift()
    print("Image test: PASS")
    display.show(Image.HAPPY)
except Exception as ae:
    display.show(Image.SAD)
    raise



'''
Conway's Game Of Life for the micro:bit

Press button A or tap the micro:bit to generate a fresh layout.
'''

import microbit
import random

arena1 = bytearray(7 * 7)
arena2 = bytearray(7 * 7)

def show():
    img = microbit.Image(5,5)
    for y in range(5):
        for x in range(5):
            img.set_pixel(x, y, arena1[8 + y * 7 + x]*9)
    microbit.display.show(img)

# do 1 iteration of Conway's Game of Life
def conway_step():
    global arena1, arena2
    for i in range(5 * 5): # loop over pixels
        i = 8 + (i // 5) * 7 + i % 5
        # count number of neighbours
        num_neighbours = (arena1[i - 8] +
                arena1[i - 7] +
                arena1[i - 6] +
                arena1[i - 1] +
                arena1[i + 1] +
                arena1[i + 6] +
                arena1[i + 7] +
                arena1[i + 8])
        # check if the centre cell is alive or not
        self = arena1[i]
        # apply the rules of life
        if self and not (2 <= num_neighbours <= 3):
            arena2[i] = 0 # not enough, or too many neighbours: cell dies
        elif not self and num_neighbours == 3:
            arena2[i] = 1 # exactly 3 neighbours around an empty cell: cell is born
        else:
            arena2[i] = self # stay as-is
    # swap the buffers (arena1 is now the new one to display)
    arena1, arena2 = arena2, arena1

while True:
    # randomise the start
    for i in range(5 * 5): # loop over pixels
        i = 8 + (i // 5) * 7 + i % 5
        arena1[i] = random.randrange(2) # set the pixel randomly
    show()
    microbit.sleep(1) # need to yield to update accelerometer (not ideal...)

    # loop while button a is not pressed
    while not microbit.button_a.is_pressed() and microbit.accelerometer.get_z() < -800:
        conway_step()
        show()
        microbit.sleep(150)



"""
    neopixel_random.py
    
    Repeatedly displays random colours onto the LED strip.
    This example requires a strip of 8 Neopixels (WS2812) connected to pin0.

"""
from microbit import *
import neopixel
from random import randint

# Setup the Neopixel strip on pin0 with a length of 8 pixels
np = neopixel.NeoPixel(pin0, 8)

while True:
    #Iterate over each LED in the strip
    
    for pixel_id in range(0, len(np)):
        red = randint(0, 60)
        green = randint(0, 60)
        blue = randint(0, 60)
        
        # Assign the current LED a random red, green and blue value between 0 and 60
        np[pixel_id] = (red, green, blue)
        
        # Display the current pixel data on the Neopixel strip
        np.show()
        sleep(100)



"""
Digital Water - your micro water simulator. By Tom Viner

Explanation and see running:
https://www.youtube.com/watch?v=OBTUjoc46Pk
"""

import microbit

# define some constants

DISPLAY_WIDTH = 5
DISPLAY_HEIGHT = 5

MIN_BRIGHTNESS = 0
MEDIUM_BRIGHTNESS = 4
MAX_BRIGHTNESS = 9

# this is how the accelerometer values 1g of gravity
ONE_G = 1024

# Some maths functions to help us

def clamp(minimum, n, maximum):
    """Return the nearest value to n, that's within minimum to maximum (incl)
    """
    return max(minimum, min(n, maximum))

def rescale(src_scale, dest_scale, x):
    """Map one number scale to another

    For example, to convert a score of 4 stars out of 5 into a percentage:
    >>> rescale((0, 5), (0, 100), 4)
    80.0

    Great for mapping different input values into LED pixel brightnesses!
    """
    src_start, src_end = src_scale
    # what proportion along src_scale x is:
    proportion = 1.0 * (x - src_start) / (src_end - src_start)

    dest_start, dest_end = dest_scale
    # apply our proportion to the dest_scale
    return proportion * (dest_end - dest_start) + dest_start

# Helpers for controling the display

def light(brightness, filter):
    """Light up all pixels matching the filter function
    """
    brightness = clamp(MIN_BRIGHTNESS, round(brightness), MAX_BRIGHTNESS)
    for col in range(DISPLAY_WIDTH):
        for row in range(DISPLAY_HEIGHT):
            if filter(col, row):
                microbit.display.set_pixel(col, row, brightness)

def fade_display():
    """Reduce every pixel by 1 brightness level

    This means as we draw new things, the old ones will fade away
    """
    for col in range(5):
        for row in range(5):
            brightness = microbit.display.get_pixel(col, row)
            # reduce by one, but make sure it's still in 0 to 9
            brightness = clamp(MIN_BRIGHTNESS, brightness - 1, MAX_BRIGHTNESS)
            microbit.display.set_pixel(col, row, brightness)


def paint_water():
    """Use the accelerometer to paint a water level on the display
    """
    # read the current orientation values from the accelerometer
    X, Y, Z = microbit.accelerometer.get_values()

    # map the force in the X-axis to a turn factor from -2 to 2
    # -ONE_G is button A at the top, ONE_G is button B at the top
    turn_factor = rescale((-ONE_G, ONE_G), (-2, 2), X)

    # map the force in the Z-axis to a spill factor from -3 to 3
    # this allows the water to cover the whole display when it's flat
    spill_factor = rescale((ONE_G, -ONE_G), (-3, 3), Z)

    # use the variables above to make a filter function, customised for the
    # current orientation of the micro:bit
    def filter(col, row):
        """For a given pixel position, decide if it should be on or not
        """
        if Y < 0:
            # we're upside down, so reverse the y-axis value
            # (- 1 because we start counting rows from 0, not 1)
            row = DISPLAY_HEIGHT - 1 - row
        # remember rows count down from the top, so we want to light up all
        # the rows below the water line (when the micro:bit is help up straight)
        # The forumula here is of the form y = m*x + c
        # We have a couple of "- 2"s to centre the water level in the middle 
        # of the display
        return row - 2 > -turn_factor * (col - 2) - spill_factor

    # we want the water to "dilute" when spread out across the whole display
    overall_brightness = rescale(
        (0, ONE_G),
        (MAX_BRIGHTNESS, MEDIUM_BRIGHTNESS),
        abs(Z)
    )

    # light up the pixels when filter returns true, to the given bright level
    light(overall_brightness, filter)

# loop forever painting watery pixels, sleeping and then fading as each pixel
# washes away into the night

while True:
    paint_water()

    microbit.sleep(100)

    # fade all pixels by one brightness level
    fade_display()



#This tests that the mode of pins can be configured properly and that pull works OK.

from microbit import *
import music

BIG_PINS = pin0, pin1, pin2
DISPLAY_PINS = pin3, pin4, pin6, pin7, pin9, pin10

def test_mode_switching():
    print ("Switching mode from analog to digital on big pins")
    for p0 in BIG_PINS:
        p0.write_analog(100)
        for p1 in BIG_PINS:
             p1.write_digital(0)
        p0.write_analog(0)
    for p in BIG_PINS:
        p.write_analog(100)
        assert p.get_mode() == "write_analog"
        p.read_digital()
        assert p.get_mode() == "read_digital"
        p.write_analog(0)
        assert p.get_mode() == "unused"
        p.write_digital(1)
        assert p.get_mode() == "write_digital"
        p.read_analog()
        assert p.get_mode() == "unused"

def test_display_pins():
    print ("Switching mode to digital for display pins")
    for p0 in DISPLAY_PINS:
        try:
            p0.write_digital(0)
        except ValueError:
            pass
        try:
            p0.write_analog(0)
        except ValueError:
            pass
    display.off()
    for p0 in DISPLAY_PINS:
        p0.write_digital(0)
    display.on()
    display.off()
    for p0 in DISPLAY_PINS:
        p0.write_analog(200)
    display.on()

def test_music():
    print ("Switching mode to digital for music pins")
    music.play(music.DADADADUM,pin=pin0,wait=False)
    try:
        pin0.write_digital(0)
    except ValueError:
        pass
    music.stop()
    pin0.write_digital(0)
    
def test_pull():
    print ("Setting pull on big pins")
    for p in BIG_PINS:
        p.read_digital()
        p.set_pull(p.PULL_UP)
        assert p.get_pull() == p.PULL_UP
        assert p.read_digital()
        p.set_pull(p.PULL_DOWN)
        assert p.get_pull() == p.PULL_DOWN
        assert p.read_digital() == 0
        p.set_pull(p.NO_PULL)
        assert p.get_pull() == p.NO_PULL
    print ("Setting pull on display pins")
    display.off()
    for p in DISPLAY_PINS:
        p.read_digital()
        p.set_pull(p.PULL_UP)
        assert p.read_digital()
        p.set_pull(p.PULL_DOWN)
        assert p.read_digital() == 0
    display.on()

try:
    test_mode_switching()
    test_display_pins()
    test_music()
    test_pull()
    print("Pin test: PASS")
    display.show(Image.HAPPY)
except Exception as ae:
    display.show(Image.SAD)
    raise




"""Four buttons - 2 buttons + 2 more "buttons"!
A little example of using pins 1 & 2 an extra 2 buttons. By Tom Viner

Explanation and see running:
https://www.youtube.com/watch?v=6ofUJ6Mgk4k
"""
import microbit

# define some constants

DISPLAY_WIDTH = 5
DISPLAY_HEIGHT = 5

MIN_BRIGHTNESS = 0
MAX_BRIGHTNESS = 9

# A maths functions to help us

def clamp(minimum, n, maximum):
    """Return the nearest value to n, that's within minimum to maximum (incl)
    """
    return max(minimum, min(n, maximum))


# Helpers for controling the display

def light(brightness, filter):
    """Light up all pixels matching the filter function
    """
    brightness = clamp(MIN_BRIGHTNESS, round(brightness), MAX_BRIGHTNESS)
    for col in range(DISPLAY_WIDTH):
        for row in range(DISPLAY_HEIGHT):
            if filter(col, row):
                microbit.display.set_pixel(col, row, brightness)

def light_column(column):
    """Light up a whole column to max brightness
    """
    def filter_column(col, row):
        """For a given pixel position, turn on if it matches our column
        """
        return col == column
    light(MAX_BRIGHTNESS, filter_column)

def light_row(row):
    """Light up a whole row to max brightness
    """
    def filter_row(col, rw):
        """For a given pixel position, turn on if it matches our row
        """
        return rw == row
    light(MAX_BRIGHTNESS, filter_row)

def fade_display():
    """Reduce every pixel by 1 brightness level

    This means as we draw new things, the old ones will fade away
    """
    for col in range(5):
        for row in range(5):
            brightness = microbit.display.get_pixel(col, row)
            # reduce by one, but make sure it's still in 0 to 9
            brightness = clamp(MIN_BRIGHTNESS, brightness - 1, MAX_BRIGHTNESS)
            microbit.display.set_pixel(col, row, brightness)

def paint_box(top=0, bottom=DISPLAY_HEIGHT-1, left=0, right=DISPLAY_WIDTH-1):
    """Draw a filled in rectangle on the display
    """
    def filter_box(col, row):
        """For a given pixel position, turn it on if it's with the bounds
        """
        # remember rows count from 0 at the top!
        correct_vertical = top <= row <= bottom
        correct_horizontal = left <= col <= right
        return correct_vertical and correct_horizontal
    light(MAX_BRIGHTNESS, filter_box)


# Our main functions

def pin_is_touched(n):
    """Pass in a pin number (1 or 2),
    get back True if it's being touched right now

    In this way, it acts just like microbit.button_a.is_pressed()
    """
    pin = getattr(microbit, 'pin{}'.format(n))
    return pin.read_analog() > 300

def four_buttons():
    """Push buttons, touch pins, see if you can light up the whole display!
    """
    a_is_pressed = microbit.button_a.is_pressed()
    b_is_pressed = microbit.button_b.is_pressed()

    # let's call the two pin-buttons c and d:
    c_is_pressed = pin_is_touched(1)
    d_is_pressed = pin_is_touched(2)

    if a_is_pressed:
        light_row(2)
        paint_box(right=1)
    if b_is_pressed:
        light_row(3)
        paint_box(left=3)
    if c_is_pressed:
        light_column(1)
        paint_box(bottom=1)
    if d_is_pressed:
        light_column(3)
        paint_box(top=3)


while True:
    four_buttons()

    microbit.sleep(10)

    # fade all pixels by one brightness level
    fade_display()



#Plays a file on the specified pins.
import audio

def audio_generator(file, frame):
    ln = -1
    while ln:
        ln = file.readinto(frame)
        yield frame

def play_file(name, pin=None, return_pin=None):
    #Do allocation here, as we can't do it in an interrupt.
    frame = audio.AudioFrame()
    with open(name) as file:
        audio.play(audio_generator(file, frame), pin=pin, return_pin=return_pin)



# Simple Slalom by Larry Hastings, September 2015
#
# This program has been placed into the public domain.

import microbit as m
import random

p = m.display.show

min_x = -1024
max_x = 1024
range_x = max_x - min_x

wall_min_speed = 400
player_min_speed = 200

wall_max_speed = 100
player_max_speed = 50

speed_max = 12


while True:

    i = m.Image('00000:'*5)
    s = i.set_pixel

    player_x = 2

    wall_y = -1
    hole = 0

    score = 0
    handled_this_wall = False

    wall_speed = wall_min_speed
    player_speed = player_min_speed

    wall_next = 0
    player_next = 0

    while True:
        t = m.running_time()
        player_update = t >= player_next
        wall_update = t >= wall_next
        if not (player_update or wall_update):
            next_event = min(wall_next, player_next)
            delta = next_event - t
            m.sleep(delta)
            continue

        if wall_update:
            # calculate new speeds
            speed = min(score, speed_max)
            wall_speed = wall_min_speed + int((wall_max_speed - wall_min_speed) * speed / speed_max)
            player_speed = player_min_speed + int((player_max_speed - player_min_speed) * speed / speed_max)

            wall_next = t + wall_speed
            if wall_y < 5:
                # erase old wall
                use_wall_y = max(wall_y, 0)
                for wall_x in range(5):
                    if wall_x != hole:
                        s(wall_x, use_wall_y, 0)

        wall_reached_player = (wall_y == 4)
        if player_update:
            player_next = t + player_speed
            # find new x coord
            x = m.accelerometer.get_x()
            x = min(max(min_x, x), max_x)
            # print("x accel", x)
            s(player_x, 4, 0) # turn off old pixel
            x = ((x - min_x) / range_x) * 5
            x = min(max(0, x), 4)
            x = int(x + 0.5)
            # print("have", position, "want", x)

            if not handled_this_wall:
                if player_x < x:
                    player_x += 1
                elif player_x > x:
                    player_x -= 1
            # print("new", position)
            # print()

        if wall_update:
            # update wall position
            wall_y += 1
            if wall_y == 7:
                wall_y = -1
                hole = random.randrange(5)
                handled_this_wall = False

            if wall_y < 5:
                # draw new wall
                use_wall_y = max(wall_y, 0)
                for wall_x in range(5):
                    if wall_x != hole:
                        s(wall_x, use_wall_y, 6)

        if wall_reached_player and not handled_this_wall:
            handled_this_wall = True
            if (player_x != hole):
                # collision! game over!
                break
            score += 1

        if player_update:
            s(player_x, 4, 9) # turn on new pixel

        p(i)

    p(i.SAD)
    m.sleep(1000)
    m.display.scroll("Score:" + str(score))

    while True:
        if (m.button_a.is_pressed() and m.button_a.is_pressed()):
            break
        m.sleep(100)



#!/usr/bin/env python

"""

This module allows files to loaded onto the Micro:Bit without
flashing the device.

Example usage:

$ ./tools/upload.py  /dev/ttyACM0 ./examples/simple_slalom.py --name main.py

Will upload the simple_slalom game into main.py
"""

import sys
import pyboard

PY2 = sys.version_info < (3,)

def main():
    import argparse
    cmd_parser = argparse.ArgumentParser(description='Load files onto the microbit.')
    cmd_parser.add_argument('-b', '--baudrate', default=115200, help='the baud rate of the serial device')
    cmd_parser.add_argument('-u', '--user', default='micro', help='the telnet login username')
    cmd_parser.add_argument('-p', '--password', default='python', help='the telnet login password')
    cmd_parser.add_argument('-w', '--wait', default=0, type=int, help='seconds to wait for USB connected board to become available')
    cmd_parser.add_argument('--name', nargs=1, help='name to save the file as, if different from the file name')
    cmd_parser.add_argument('device', nargs=1, help='the serial device or the IP address of the microbit')
    cmd_parser.add_argument('file', nargs=1, help='file to load')
    args = cmd_parser.parse_args()

    def exec_commands(cmds):
        try:
            pyb = pyboard.Pyboard(args.device[0], args.baudrate, args.user, args.password, args.wait)
            pyb.enter_raw_repl()
            for cmd in cmds:
                ret, ret_err = pyb.exec_raw(cmd, timeout=None, data_consumer=pyboard.stdout_write_bytes)
                if ret_err:
                    break
            pyb.exit_raw_repl()
            pyb.close()
        except Exception as er:
            print(er)
            sys.exit(1)
        except KeyboardInterrupt:
            sys.exit(1)
        if ret_err:
            pyboard.stdout_write_bytes(ret_err)
            sys.exit(1)

    with open(args.file[0], 'rb') as f:
        pyfile = f.read()
        if args.name:
            name = args.name[0]
        else:
            name = args.file[0]
        script = make_save_script(pyfile, name)
        exec_commands(script)

def make_save_script(pyfile, filename):
    '''Convert a file into a script that saves that file
    into the persistent script area on the microbit'''
    output = [
        '''
fd = open("%s", "wb")
f = fd.write''' % filename
    ]
    while pyfile:
        line = pyfile[:64]
        if PY2:
            output.append('f(b' + repr(line) + ')')
        else:
            output.append('f(' + repr(line) + ')')
        pyfile = pyfile[64:]
    output.append('fd.close()')
    return output

if __name__ == "__main__":
    main()



# test speech module

import microbit
import speech

def test_funcs():
    print('Testing basic functions')
    ph = speech.translate('hello world')
    assert ph == ' /HEHLOW WERLD'
    speech.pronounce('PIHTHUN', pitch=32, speed=60, mouth=100, throat=150)
    speech.say('hello')
    speech.sing('YEHSTERDEY5')

def test_sleep():
    # check that sleep works ok with speech because they both use low-level timers
    # (this is really a test of the audio module)
    print('Testing sleep with speech')
    microbit.sleep(1)
    speech.say('hello world')
    microbit.sleep(1)

def test_timing():
    # test that speech takes the correct amount of time over many runs
    print('Testing timing of say function')
    for i in range(5):
        start = microbit.running_time()
        speech.say('hello world')
        microbit.sleep(1)
        stop = microbit.running_time()
        assert 800 < stop - start < 815

try:
    test_funcs()
    test_sleep()
    test_timing()
    print('Speech test: PASS')
except Exception as ae:
    raise



# A micro:bit Firefly.
# By Nicholas H.Tollervey. Released to the public domain.
import radio
import random
from microbit import display, Image, button_a, sleep

# Create the "flash" animation frames. Can you work out how it's done?
flash = [Image().invert()*(i/9) for i in range(9, -1, -1)]

# The radio won't work unless it's switched on.
radio.on()

# Event loop.
while True:
    # Button A sends a "flash" message.
    if button_a.was_pressed():
        radio.send('flash')  # a-ha
    # Read any incoming messages.
    incoming = radio.receive()
    if incoming == 'flash':
        # If there's an incoming "flash" message display
        # the firefly flash animation after a random short
        # pause.
        sleep(random.randint(50, 350))
        display.show(flash, delay=100, wait=False)
        # Randomly re-broadcast the flash message after a
        # slight delay.
        if random.randint(0, 9) == 0:
            sleep(500)
            radio.send('flash')  # a-ha



# Magic 8 ball by Nicholas Tollervey. February 2016.
#
# Ask a question then shake.
#
# This program has been placed into the public domain.
from microbit import *
import random

answers = [
    "It is certain",
    "It is decidedly so",
    "Without a doubt",
    "Yes, definitely",
    "You may rely on it",
    "As I see it, yes",
    "Most likely",
    "Outlook good",
    "Yes",
    "Signs point to yes",
    "Reply hazy try again",
    "Ask again later",
    "Better not tell you now",
    "Cannot predict now",
    "Concentrate and ask again",
    "Don't count on it",
    "My reply is no",
    "My sources say no",
    "Outlook not so good",
    "Very doubtful",
]

while True:
    display.show('8')
    if accelerometer.was_gesture('shake'):
        display.clear()
        sleep(1000)
        display.scroll(random.choice(answers))
    sleep(10)



import audio

def from_file(file, frame):
    ln = -1
    while ln:
        ln = file.readinto(frame)
        yield frame

def reverb_gen(src, buckets, reflect, fadeout):
    bucket_count = len(buckets)
    bucket = 0
    for frame in src:
        echo = buckets[bucket]
        echo *= reflect
        echo += frame
        yield echo
        buckets[bucket] = echo
        bucket += 1
        if bucket == bucket_count:
            bucket = 0
    while fadeout:
        fadeout -= 1
        echo = buckets[bucket]
        echo *= reflect
        yield echo
        buckets[bucket] = echo
        bucket += 1
        if bucket == bucket_count:
            bucket = 0

def reverb(src, delay, reflect):
    #Do all allocation up front, so we don't need to do any in the generator.
    bucket_count = delay>>2
    buckets = [ None ] * bucket_count
    for i in range(bucket_count):
        buckets[i] = audio.AudioFrame()
    vol = 1.0
    fadeout = 0
    while vol > 0.05:
        fadeout += bucket_count
        vol *= reflect
    return reverb_gen(src, buckets, reflect, fadeout)

def play_file(name, delay=80, reflect=0.5):
    #Do allocation here, as we can't do it in an interrupt.
    frame = audio.AudioFrame()
    with open(name) as file:
        gen = from_file(file, frame)
        r = reverb(gen, delay, reflect)
        audio.play(r)



from microbit import *

hands = Image.ALL_CLOCKS

#A centre dot of brightness 2.
ticker_image = Image("2\n").crop(-2,-2,5,5)

#Adjust these to taste
MINUTE_BRIGHT = 0.1111
HOUR_BRIGHT = 0.55555

#Generate hands for 5 minute intervals
def fiveticks():
    fivemins = 0
    hours = 0
    while True:
        yield  hands[fivemins]*MINUTE_BRIGHT + hands[hours]*HOUR_BRIGHT
        fivemins = (fivemins+1)%12
        hours = (hours + (fivemins == 0))%12

#Generate hands with ticker superimposed for 1 minute intervals.      
def ticks():
    on = True
    for face in fiveticks():
        for i in range(5):
            if on:
                yield face + ticker_image
            else:
                yield face - ticker_image
            on = not on

#Run a clock speeded up 60 times, so we can watch the animation.
for tick in ticks():
    display.show(tick)
    sleep(1000)



import speech
from microbit import sleep

# The say method attempts to convert English into phonemes.
speech.say("I can sing!")
sleep(1000)
speech.say("Listen to me!")
sleep(1000)

# Clearing the throat requires the use of phonemes. Changing
# the pitch and speed also helps create the right effect.
speech.pronounce("AEAE/HAEMM", pitch=200, speed=100)  # Ahem
sleep(1000)

# Singing requires a phoneme with an annotated pitch for each syllable.
solfa = [
    "#115DOWWWWWW",   # Doh
    "#103REYYYYYY",   # Re
    "#94MIYYYYYY",    # Mi
    "#88FAOAOAOAOR",  # Fa
    "#78SOHWWWWW",    # Soh
    "#70LAOAOAOAOR",  # La
    "#62TIYYYYYY",    # Ti
    "#58DOWWWWWW",    # Doh
]

# Sing the scale ascending in pitch.
song = ''.join(solfa)
speech.sing(song, speed=100)
# Reverse the list of syllables.
solfa.reverse()
song = ''.join(solfa)
# Sing the scale descending in pitch.
speech.sing(song, speed=100)




import os
from microbit import display, Image

#We don't have space for Beowolf, so here's the next best thing...
text = """
’Twas brillig, and the slithy toves
  Did gyre and gimble in the wabe:
All mimsy were the borogoves,
  And the mome raths outgrabe.

“Beware the Jabberwock, my son!
  The jaws that bite, the claws that catch!
Beware the Jubjub bird, and shun
 The frumious Bandersnatch!”

He took his vorpal sword in hand;
  Long time the manxome foe he sought—
So rested he by the Tumtum tree
  And stood awhile in thought.

And, as in uffish thought he stood,
  The Jabberwock, with eyes of flame,
Came whiffling through the tulgey wood,
  And burbled as it came!

One, two! One, two! And through and through
  The vorpal blade went snicker-snack!
He left it dead, and with its head
  He went galumphing back.

“And hast thou slain the Jabberwock?
  Come to my arms, my beamish boy!
O frabjous day! Callooh! Callay!”
  He chortled in his joy.

’Twas brillig, and the slithy toves
  Did gyre and gimble in the wabe:
All mimsy were the borogoves,
  And the mome raths outgrabe.
"""

def test_read_while_writing():
    j1 = open("jabbawocky.txt", "w")
    j1.write(text)
    j2 = open("jabbawocky.txt")
    short = j2.read()
    assert text.startswith(short)
    del j2
    j1.close()
    j2 = open("jabbawocky.txt")
    assert j2.read() == text
    j2.close()
    os.remove("jabbawocky.txt")

def test_removing_mid_read():
    with open("jabbawocky.txt", "w") as j:
        j.write(text)
    j = open("jabbawocky.txt")
    os.remove("jabbawocky.txt")
    try:
        j.read()
        assert False, "Shouldn't reach here"
    except OSError:
        pass

def test_removing_mid_write():
    j = open("jabbawocky.txt", "w")
    os.remove("jabbawocky.txt")
    try:
        j.write(text)
        assert False, "Shouldn't reach here"
    except OSError:
        pass

def test_repeated_write():
    for i in range(40):
        with open("jabbawocky.txt", "w") as j:
            j.write(text)

display.clear()
try:
    test_read_while_writing()
    test_removing_mid_read()
    test_removing_mid_write()
    test_repeated_write()
    print("File test: PASS")
    display.show(Image.HAPPY)
except Exception as ae:
    display.show(Image.SAD)
    raise





import os
from microbit import display, Image


def data_stream(seed):
    val = seed&0xffff
    A = 33
    C = 509
    while True:
        val = (A*val+C)&0xffff
        yield val >> 8

def text_stream(seed):
    val = seed&0xffff
    A = 33
    C = 509
    while True:
        val = (A*val+C)&0xffff
        yield chr(val >> 10 + 34)

def write_data_to_file(name, seed, n, count):
    print ("Writing data to " + name)
    buf = bytearray(n)
    d = data_stream(seed)
    with open(name, 'wb') as fd:
        for i in range(count):
            for j in range(n):
                buf[j] = next(d)
            fd.write(buf)

def verify_file(name, stream, n, count, mode):
    print ("Verifying data in " + name)
    byte_count = 0
    with open(name, mode) as fd:
        while True:
            buf = fd.read(n)
            assert len(buf) <= n
            if not buf:
                break
            for b in buf:
                expected = next(stream)
                assert b == expected, "expected %d, got %d at %d" % (expected, b, byte_count)
                byte_count += 1
    assert byte_count == n*count, "expected count of %d, got %d" % (n*count, byte_count)

def clear_files():
    for f in os.listdir():
        os.remove(f)

def test_interleaved_small_files():
    for i in range(80):
        name = "%d.dat" % i
        write_data_to_file(name, i*3, 16, 6)
    for i in range(0, 80, 2):
        os.remove("%d.dat" % i)
    for i in range(80, 120):
        name = "%d.dat" % i
        write_data_to_file(name, i*3, 16, 6)
        verify_file(name, data_stream(i*3), 16, 6, 'b')
    for i in range(1, 80, 2):
        os.remove("%d.dat" % i)
    for i in range(80, 120):
        os.remove("%d.dat" % i)
    assert not os.listdir()

def test_interleaved_large_files():
    out_buf = bytearray(100)
    for i in range(100):
        out_buf[i] = 100-i
    with open("test1.dat", "wb") as fd1:
        with open("test2.dat", "wb") as fd2:
            for i in range(60):
                fd1.write(out_buf)
                fd2.write(out_buf)
    os.remove("test2.dat")
    with open("test3.dat", "wb") as fd3:
        for i in range(60):
            fd3.write(out_buf)
    assert sorted(os.listdir()) == [ "test1.dat", "test3.dat" ]
    in_buf = bytearray(100)
    with open("test1.dat", "rb") as fd:
        for i in range(60):
            fd.readinto(in_buf)
            assert in_buf == out_buf
    with open("test3.dat", "rb") as fd:
        for i in range(60):
            fd.readinto(in_buf)
            assert in_buf == out_buf

display.clear()
try:
    clear_files()
    test_interleaved_small_files()
    test_interleaved_large_files()
    print("File test: PASS")
    display.show(Image.HAPPY)
except Exception as ae:
    display.show(Image.SAD)
    raise



"""
Program that shows different emotions.
Push button "A" to become sadder and "B" to become happier.
"""

from microbit import *

horror = Image("09090:00000:09990:90009:99999")
better_meh = Image("00000:09090:00000:99999:00000")
joy = Image("09090:00000:99999:90009:09990")

emotions = [horror, Image.SAD, better_meh, Image.HAPPY, joy]
current_emotion = 2

while True:
    if button_a.get_presses():
        current_emotion = max(current_emotion - 1, 0)
    elif button_b.get_presses():
        current_emotion = min(current_emotion + 1, 4)
    display.show(emotions[current_emotion])


### Flame simulation on the Microbit.
### Author: M. Schafer 2016
# This program has been placed into the public domain.

import microbit, random

# User adjustable values for range of brightness in flames.
MIN_BRIGHTNESS = 1
MAX_BRIGHTNESS = 8


#  fixed for the Microbit
DISPLAY_WIDTH  = 5
DISPLAY_HEIGHT = 5

INVERT_DISPLAY = True # flame can be oriented in either direction

# MASK to create fire shape. multiplies values %
MASK = [[ 88, 100, 100, 100, 88 ],
        [ 60,  95, 100,  95, 60 ],
        [ 50,  88,  90,  88, 50 ],
        [ 33,  75,  88,  75, 33 ],
        [ 10,  33,  66,  33, 10 ]  ]

# Generate a new bottom row of random values for the flames
def generate_line(start=MIN_BRIGHTNESS, end=MAX_BRIGHTNESS):
    "start and end define range of dimmest to brightest 'flames'"
    return [start + random.randrange(end-start) for i in range(DISPLAY_WIDTH)]

# shift all values in the grid up one row
def shift_up(grid, newline):
    "Shift up lines in grid, add newline at bottom"
    for y in range(DISPLAY_HEIGHT-1, 0, -1):
        grid[y] = grid[y-1]
    # lowest line
    for x in range(DISPLAY_WIDTH):
        grid[0] = newline


# write a frame to the screen.
# Interpolate values based on percent
def interpolate_frame(screen, pcnt, grid, line):
    """ Interpolate new values by reading from grid and
         writing to the screen """
    # each row interpolates with the one before it
    for y in range(DISPLAY_HEIGHT-1, 0, -1):
        for x in range(DISPLAY_WIDTH):
            mask = MASK[y][x]
            newval = ((100-pcnt) * grid[y][x] + pcnt * grid[y-1][x] ) / 100.0
            newval = mask * newval / 100.0
            if INVERT_DISPLAY:
                screen.set_pixel(x, DISPLAY_HEIGHT-y-1, int(newval))
            else:
                screen.set_pixel(x, y, int(newval))
    # first row interpolates with the "next" line
    for x in range(DISPLAY_WIDTH):
        mask = MASK[y][x]
        newval = ((100-pcnt) * grid[0][x] + pcnt * line[x]) / 100.0
        newval = mask * newval / 100.0
        if INVERT_DISPLAY:
            screen.set_pixel(x, DISPLAY_HEIGHT-1, int(newval))
        else:
            screen.set_pixel(x, 0, int(newval))

## Setup

line = generate_line()
grid = [[0 for i in range(DISPLAY_WIDTH)] for i in range(DISPLAY_HEIGHT)]

SCREEN = microbit.display
percent = 0     # counter to see when to re-interpolate
sleeptime = 0   # delay between updates
percent_increment = 25  # how fast we interpolate fire


# loop forever
while True:
    if percent > 100:
        # move everything up a line, insert new bottom row
        line = generate_line()
        shift_up(grid, line)
        percent = 0

    # Check Buttons to see if changing
    # button_a = smoothness
    if microbit.button_a.was_pressed():
        percent_increment += 5
        if percent_increment > 50:
             percent_increment = 1
        print("percent interpolate=", percent_increment)
    # button_b = delay
    if microbit.button_b.was_pressed():
        sleeptime += 10
        if sleeptime > 100:
             sleeptime = 0
        print("sleeptime=", sleeptime)
    # draw frame and sleep
    interpolate_frame(SCREEN, percent, grid, line)
    microbit.sleep(sleeptime)
    # update main counters
    percent += percent_increment




# test the random module

import random

result = ('FAIL', 'PASS')

# test getrandbits
hist = 8 * [0]
for i in range(300):
    r = random.getrandbits(3)
    hist[r] += 1
ok = all(h >= 10 for h in hist)
print('getrandbits:', result[ok])

# test seed
random.seed(0x1234567)
r1 = random.getrandbits(10)
random.seed(0x1234567)
r2 = random.getrandbits(10)
print('seed:       ', result[r1 == r2])

# test randint is within the given range
ok = True
for i in range(100):
    if not 0 <= random.randint(0, 9) <= 9:
        ok = False
print('randint:    ', result[ok])



#!/usr/bin/env python3

'''
Combine the MicroPython firmware with a Python script and produce a hex file
ready for uploading to the micro:bit.

Usage: ./makecombinedhex.py <firmware.hex> <script.py> [-o <combined.hex>]

Output goes to stdout if no filename is given.
'''

import sys
import argparse
import hexlifyscript

def get_largest_addr(hexfile):
    largest_addr = 0
    for line in hexfile:
        count = int(line[1:3], 16)
        addr = int(line[3:7], 16)
        type = int(line[7:9], 16)
        if count == 2 and type == 4: # ext linear addr
            page = int(line[9:13], 16) << 16
        elif type == 0: # data
            # only count pages in flash, not in the UICR
            if page < 0x10000000:
                largest_addr = max(largest_addr, page + addr + count)
    return largest_addr

def find_uicr_line(hexfile):
    for i, line in enumerate(hexfile):
        # UICR from 0x10001000 so we expect an extended linear address record
        if ':020000041000EA' in line:
            return i
    return None

if __name__ == '__main__':
    arg_parser = argparse.ArgumentParser(description='Produce combined hex firmware for the micro:bit.')
    arg_parser.add_argument('-o', '--output', default=sys.stdout, type=argparse.FileType('wt'), help='output file (default is stdout)')
    arg_parser.add_argument('firmware', nargs=1, help='input MicroPython firmware')
    arg_parser.add_argument('script', nargs=1, help='input Python script')
    args = arg_parser.parse_args()

    # read in the firmware
    with open(args.firmware[0], 'rt') as f:
        firmware = f.readlines()

    # check the firmware is not too large
    if get_largest_addr(firmware) > hexlifyscript.SCRIPT_ADDR:
        raise Exception('firmware overflows into script region')

    # hexlify the script
    with open(args.script[0], 'rb') as f:
        script = hexlifyscript.hexlify_script(f.read())

    # print lines until UICR area or the start linear address record
    firmware_end = find_uicr_line(firmware) or len(firmware) - 2
    for line in firmware[:firmware_end]:
        print(line, end='', file=args.output)

    # print script
    print(script, file=args.output)

    # print rest of hex file
    for line in firmware[firmware_end:]:
        print(line, end='', file=args.output)



# Light LEDs at random and make them fade over time
#
# Usage:
#
#    led_dance(delay)
#
# 'delay' is the time between each new LED being turned on.
#
# TODO The random number generator is not great. Perhaps the accelerometer
# or compass could be used to add entropy.

import microbit
import random

def led_dance(delay):
    dots = [ [0]*5, [0]*5, [0]*5, [0]*5, [0]*5 ]
    while True:
        dots[random.randrange(5)][random.randrange(5)] = 8
        for i in range(5):
            for j in range(5):
                microbit.display.set_pixel(i, j, dots[i][j])
                dots[i][j] = max(dots[i][j] - 1, 0)
        microbit.sleep(delay)

led_dance(100)



'''
Flappy Bit

Control the bit by tilting the micro:bit

Avoid the obstacles

Create your own terrain by editing the terrain list below:
'''

import music
from microbit import (accelerometer,
                      display,
                      sleep,
                      Image,
                      reset,
                      )

display.scroll('Flappy Bit')

bird = 2
terrain = [
    (0, 0),
    (0, 0),
    (0, 0),
    (0, 0),
    (0, 2),
    (0, 0),
    (0, 0),
    (3, 0),
    (3, 0),
    (4, 0),
    (3, 0),
    (0, 0),
    (0, 0),
    (0, 1),
    (2, 0),
    (3, 0),
    (4, 0),
    (3, 0),
    (0, 0),
    (0, 1),
    (2, 0),
]

terrain_multiplier = 5
pos = 0

while True:
    sleep(100)
    if -256 < accelerometer.get_y() < 450:
        bird = max(0, bird - 1)
    elif 568 < accelerometer.get_y() < 1024:
        bird = min(4, bird + 1)

    display.clear()
    display.set_pixel(0, bird, 9)

    pos_terrain = pos // terrain_multiplier
    lost_status = False
    for column, (top, bottom) in enumerate(
            terrain[pos_terrain:pos_terrain + 5]):
        for y in range(top):
            display.set_pixel(column, y, 4)
            if column == 0 and bird == y:
                lost_status = True
        for y in range(bottom):
            display.set_pixel(column, 4 - y, 4)
            if column == 0 and bird == (4 - y):
                lost_status = True
    if lost_status:
        display.show(Image.SAD)
        music.play(music.FUNERAL)
        reset()
    pos += 1
    if pos_terrain > len(terrain):
        pos = 0



#!/usr/bin/env python3

'''
Add a UICR region to the hex firmware for MicroPython on the micro:bit.

Usage: ./adduicr.py <firmware.hex> <version_string_address> [-o <combined.hex>]

Output goes to stdout if no filename is given.
'''

import sys
import struct
import argparse
import hexlifyscript
import makecombinedhex

NRF_PAGE_SIZE_LOG2 = 10
NRF_PAGE_SIZE = 1 << NRF_PAGE_SIZE_LOG2
UICR_BASE_ADDR = 0x100010c0
UICR_MAGIC_NUMBER = 0x17eeb07c

if __name__ == '__main__':
    arg_parser = argparse.ArgumentParser(description='Add UICR region to hex firmware for the micro:bit.')
    arg_parser.add_argument('-o', '--output', default=sys.stdout, type=argparse.FileType('wt'), help='output file (default is stdout)')
    arg_parser.add_argument('firmware', nargs=1, help='input MicroPython firmware')
    arg_parser.add_argument('address', nargs=1, type=lambda x: int(x, 0), help='address in flash of the version string')
    args = arg_parser.parse_args()

    # read in the firmware
    with open(args.firmware[0], 'rt') as f:
        firmware = f.readlines()

    # print head of firmware
    for line in firmware[:-2]:
        print(line, end='', file=args.output)

    # make UICR data
    uicr_data = b''
    uicr_data += struct.pack('<III', UICR_MAGIC_NUMBER, 0xffffffff, NRF_PAGE_SIZE_LOG2)
    uicr_data += struct.pack('<HHII', 0, (makecombinedhex.get_largest_addr(firmware) + NRF_PAGE_SIZE - 1) // NRF_PAGE_SIZE, 0xffffffff, args.address[0])
    uicr_data += struct.pack('<I', 0)

    # print UICR data
    print(hexlifyscript.make_ihex_record(0, hexlifyscript.IHEX_TYPE_EXT_LIN_ADDR, struct.pack('>H', UICR_BASE_ADDR >> 16)), file=args.output)
    for i in range(0, len(uicr_data), 16):
        chunk = uicr_data[i:min(i + 16, len(uicr_data))]
        print(hexlifyscript.make_ihex_record(UICR_BASE_ADDR + i, hexlifyscript.IHEX_TYPE_DATA, chunk), file=args.output)

    # print tail of firmware
    print(firmware[-2], end='', file=args.output)
    print(firmware[-1], end='', file=args.output)




from microbit import display, Image, pin0, pin1, pin2, pin8
import music

PINS = pin0, pin1, pin2, pin8
FREQS = 400, 500, 600, 700, 800, 900

def test_rapid_pin_switch():
    for i in range(20):
        for pin in PINS:
            music.play(music.NYAN, pin, wait=False)

def test_rapid_pitch_switch():
    for i in range(20):
        for freq in FREQS:
            music.pitch(freq, wait=False)

def test_repeated_stop():
    for i in range(20):
        music.stop()

def test_repeated_reset():
    for i in range(20):
        music.reset()

def test_repeated_set_tempo():
    for i in range(20):
        music.set_tempo(ticks=i%4+1, bpm=i+100)

def test_all_pins_free():
    for pin in PINS:
        pin.read_digital()

display.clear()
try:
    test_rapid_pin_switch()
    test_rapid_pitch_switch()
    test_repeated_stop()
    test_repeated_reset()
    test_repeated_set_tempo()
    test_all_pins_free()
    print("File test: PASS")
    display.show(Image.HAPPY)
except Exception as ae:
    display.show(Image.SAD)
    raise



"""
A simple pomodoro timer.
It times a 25 minute work session then 5 minutes rest.
Press the reset button to restart the timer.
"""

from microbit import *

# Tweak CLOCK_ADJUST to make your system clock more accurate.
# My clock is too fast by 4 seconds every minute so I use 4/60.
# If your clock is too slow by 3 seconds every minute use -3/60.

CLOCK_ADJUST = 4/60 

ALL_LEDS_ON = Image('99999:'*5)


def index_to_xy(i):
    x = i % 5
    y = int(i / 5)
    return x, y    


def show_alarm():
    for i in range(10):
        display.show(ALL_LEDS_ON)
        sleep(250)
        display.clear()
        sleep(250)


def run_timer(seconds, LED_state):
    interval = int(seconds * 1000 / 25 * (1 + CLOCK_ADJUST))
    intervals_remaining = 25
    timer = running_time()

    while intervals_remaining > 0:

        # Every interval set a pixel to LED_state
        time = running_time()
        if time - timer >= interval:
            timer = time
            x, y = index_to_xy(intervals_remaining - 1)
            display.set_pixel(x, y, LED_state)
            intervals_remaining -= 1


print("pomodoro timer")


# time the pomodoro work session, 25 minutes
display.scroll("Go!")
display.show(ALL_LEDS_ON)
run_timer(25 * 60, 0)
show_alarm()


# time the pomodoro break, 5 minutes
display.scroll("break")
display.clear()
run_timer(5 * 60, 1)
show_alarm()

display.show(Image.NO)

print("finished\n")
print("Press the reset button to restart timer.")




import audio
import radio
from microbit import button_a, button_b, display, running_time, sleep
import os

def sample_generator(filename):
    buf = audio.AudioFrame()
    with open(filename, "rb") as file:
        ln = -1
        while ln:
            ln = file.readinto(buf)
            yield buf

# 1 second of 128Hz sawtooth wave.
def sawtooth_generator():
    sawtooth = audio.AudioFrame()
    for i in range(32):
        sawtooth[i] = i*8+4
    for i in range(256):
        yield sawtooth


def send():
    display.clear()
    radio.on()
    radio.config(channel=90, power=4)
    if "sample.raw" in os.listdir():
        gen = sample_generator("sample.raw")
    else:
        gen = sawtooth_generator()
    start = running_time()
    sent = 0
    for f in gen:
        # One frame every 4ms = 8kHz
        while sent > ((running_time() - start) >> 2) + 3:
            sleep(1)
        radio.send_bytes(f)
        sent += 1
    print(sent)

def play():
    display.clear()
    radio.on()
    radio.config(channel=90, queue=12)
    count = -1
    def gen():
        recvd = audio.AudioFrame()
        empty = audio.AudioFrame()
        while True:
            if radio.receive_bytes_into(recvd) == 32:
                yield recvd
            else:
                yield empty
            if button_a.is_pressed() and button_b.is_pressed():
                return
    audio.play(gen())

while True:
    message = "Press button a to send 'sample.raw' or sawtooth wave. Press button b to play received waveform. Press both buttons to stop."
    display.scroll(message, delay=100, wait=False)
    message_end = running_time() + len(message)*600
    if button_a.is_pressed() and button_b.is_pressed():
        break
    while True:
        sleep(50)
        if button_a.is_pressed():
            send()
            break
        if button_b.is_pressed():
            play()
            break
        if running_time() > message_end:
            break



"""
This script uses the inline assembler to make the LEDs light up
in a pattern based on how they are multiplexed in rows/cols.
"""

# row pins: 13, 14, 15
# col pins: 4..12 inclusive
# GPIO words starting at 0x50000500:
#   RESERVED, OUT, OUTSET, OUTCLR, IN, DIR, DIRSET, DIRCLR

@micropython.asm_thumb
def led_cycle():
    b(START)

    # DELAY routine
    label(DELAY)
    mov(r3, 0xa0)
    lsl(r3, r3, 11)
    label(delay_loop)
    sub(r3, 1)
    bne(delay_loop)
    bx(lr)

    label(START)

    cpsid('i')          # disable interrupts so we control the display

    mov(r0, 0x50)       # r0=0x50
    lsl(r0, r0, 16)     # r0=0x500000
    add(r0, 0x05)       # r0=0x500005
    lsl(r0, r0, 8)      # r0=0x50000500 -- this points to GPIO registers
    mov(r1, 0b111)
    lsl(r1, r1, 13)     # r1=0xe000
    str(r1, [r0, 8])    # pull all rows high

    mov(r1, 1 << 4)     # r1 holds current col bit
    mov(r2, 9)          # r2 holds number of cols left
    label(loop_on)
    str(r1, [r0, 12])   # pull col low to turn LEDs on
    bl(DELAY)           # wait
    lsl(r1, r1, 1)      # shift to next col
    sub(r2, 1)          # decrease col counter
    bne(loop_on)        # loop while there are still cols left

    mov(r1, 1 << 4)     # r1 holds current col bit
    mov(r2, 9)          # r2 holds number of cols left
    label(loop_off)
    str(r1, [r0, 8])    # pull col high to turn LEDs off
    bl(DELAY)           # wait
    lsl(r1, r1, 1)      # shift to next col
    sub(r2, 1)          # decrease col counter
    bne(loop_off)       # loop while there are still cols left

    cpsie('i')      # enable interrupts

for i in range(4):
    led_cycle()



# Exercises the micro:bit - NOT EXHAUSTIVE!
from microbit import *
import music
import random


# Press A to start.
while True:
    if button_a.was_pressed():
        break
    else:
        display.show(Image.ARROW_W)
        sleep(200)
        display.clear()
        sleep(200)


# Asyncronously play a jolly little tune (connect speaker to pin0 and GND)
music.play(music.NYAN, wait=False)


# Grab all the built in images.
images = [getattr(Image, img) for img in dir(Image)
          if type(getattr(Image, img)) == Image]
# ... and cycle through them on the display.
pause = 1000
for img in images:
    display.show(img)
    sleep(pause)
    pause -= 50
    if pause < 100:
        pause = 100
display.clear()


# Aural testing of the accelerometer.
display.scroll("Accelerometer")
display.show("X")
while not button_a.is_pressed():
    music.pitch(abs(accelerometer.get_x()), 20)
sleep(500)
display.show("Y")
while not button_a.is_pressed():
    music.pitch(abs(accelerometer.get_y()), 20)
sleep(500)
display.show("Z")
while not button_a.is_pressed():
    music.pitch(abs(accelerometer.get_z()), 20)


# Aural testing of the compass.
display.scroll("Compass")
compass.calibrate()
while not button_b.is_pressed():
    music.pitch(abs(compass.heading()), 20)


# Pixel brightness.
display.scroll("Display")
dots = [ [0]*5, [0]*5, [0]*5, [0]*5, [0]*5 ]
while not button_a.is_pressed():
    dots[random.randrange(5)][random.randrange(5)] = 9
    for i in range(5):
        for j in range(5):
            display.set_pixel(i, j, dots[i][j])
            dots[i][j] = max(dots[i][j] - 1, 0)
    sleep(50)


# ??? Add further tests here...


# Finished!
display.scroll("Finished!")
display.show(Image.HAPPY)



# TiltMusic by Alex "Chozabu" P-B. September 2016.
#
# Tilt Y to change Pitch
# press A to turn sound on or off
# hold B and tilt X to change the note length
#
# A quick demo can be found at https://youtu.be/vvECQTDiWxQ
#
# This program has been placed into the public domain.

from microbit import *
import music

#A selection of sharp notes
notes = [233.08, 277.18, 311.13, 369.99, 415.30,
466.16, 554.37, 622.25, 739.99, 830.61, 932.33,
1108.73, 1244.51, 1479.98, 1661.22, 1864.66,
2217.46, 2489.02, 2959.96, 3322.44, 3729.31,
4434.92, 4978.03, 5919.91, 6644.88, 7458.62]

#note lengths
note_durations = [
 50, 100, 200, 400, 800
]
durationlen = len(note_durations)
notelen = len(notes)

duration = 100

play_music = True

while True:
    #get accelerometer readings
    xreading = abs(accelerometer.get_x())
    yreading = abs(accelerometer.get_y())
    
    #use a to toggle music
    if button_a.was_pressed():
        play_music = not play_music
    if not play_music:
        continue
    
    #get a note based on tilt
    note = xreading*.01
    pitch = notes[int(note)%notelen]
    
    #if b is pressed, alter the length based on tilt
    if button_b.is_pressed() == 1:
        #pitch *= .5
        duration = note_durations[int(yreading*0.01)%durationlen]
    
    #play our sound!
    music.pitch(int(pitch), duration)
        



"""
    counter.py
    ~~~~~~~~~~
    Creates a counter that increments on pressing button a.
    Scrolls the current count on the led display.
"""

import microbit
ctr = 0                                         # Initialize counter
while True:                                     # Loop forever
    ctr = ctr + microbit.button_a.get_presses() # Count the amount of presses 
    if ctr > 0:                                 # Only execute if not zero
        microbit.display.scroll(str(ctr))       # Display the counter
    microbit.sleep(100)                         # Sleep for 100ms



"""
    compass.py
    ~~~~~~~~~~

    Creates a compass.

    The user will need to calibrate the compass first. The compass uses the
    built-in clock images to display the position of the needle.

"""
from microbit import *


# Start calibrating
compass.calibrate()

# Try to keep the needle pointed in (roughly) the correct direction
while True:
    sleep(100)
    needle = ((15 - compass.heading()) // 30) % 12
    display.show(Image.ALL_CLOCKS[needle])



######################################################
# A watch (as in a small clock for your wrist or pocket)
# 
# Button A sets the mode: Clock or Setting time
# Button B 
#     in clock mode: shows the time as a scrolling display 
#     in setting mode: increments the time
# 
# The LED array displays the clock time in the format hh:mm. 
# The digits of the time are represented by columns of LEDs.
# 
# The digits 1 - 5 are represented by more LEDs being lit from 
# the bottom up.
# 
# For instance the digit 3  would look like:
# 
#    .
#    .
#    X
#    X
#    X
# 
# 
# The digits 6 - 9 are represented by LEDs being turned off from
# the bottom up. The digit 6 would look like:
# 
#    X
#    X
#    X
#    X
#    .
# 
# The centre column is a colon flashing once a second to separate hours from minutes.
# 
# The time 17:49 would look like:
# 
#    . X . . X
#    . X . X .
#    . X . X .
#    . . . X .
#    X . . X .
# 
# 
######################################################

from microbit import *

# Tweak CLOCK_ADJUST to make your system clock more accurate.
# My clock is too fast by 4 seconds every minute so I use 4/60.
# If your clock is too slow by 3 seconds every minute use -3/60.
CLOCK_ADJUST = 4/60 

last_button_a_state = False
last_button_b_state = False
last_display_time = 0
base_time = 0
mode = 0

modes = {0:"clock", 1:"set h", 2:"mx10", 3:"m"}


def decode_time(milliseconds):
    """Converts a time in milliseconds into a string with hours:minutes,"""
    mins = int(milliseconds / (1000 * 60) % 60)
    hrs = int(milliseconds / (1000 * 60 * 60) % 24)
    return "{h:0>2}:{m:0>2}".format(h=hrs, m=mins)


def show_time(time):
    time_string = decode_time(time)

    for i in range(5):
        if time_string[i].isdigit():
            d  = int(time_string[i])
            plot_LED_column(i, d)

    show_colon(mode==0 and int((time / 1000) % 2))


def show_colon(visible):
    display.set_pixel(2, 1, visible*9)
    display.set_pixel(2, 3, visible*9)


def get_clock_time():
    global base_time
    sys_time = running_time() / (1 + CLOCK_ADJUST)
    time = (sys_time - base_time) % (24 * 60 * 60 * 1000)
    base_time = sys_time - time 
    return time


def plot_LED_column(column, number):
    """plots a column of LEDs to represent a number from 0 - 9"""
    if number > 9:
        number = 9

    if number <= 5:
        for i in range(4, -1, -1):
            if i < 5 - number:
                display.set_pixel(column, i, 0)
            else:
                display.set_pixel(column, i, 9)

    if number > 5:
        for i in range(4, -1, -1):
            if i < 5 - (number - 5):
                display.set_pixel(column, i, 9)
            else:
                display.set_pixel(column, i, 0)


while True:
    # detect a change in button A's state, the Mode button
    button_a_state = button_a.is_pressed()
    if button_a_state != last_button_a_state:
        last_button_a_state = button_a_state

        #increment the mode
        if button_a_state == True:
            mode = (mode + 1) % 4
            display.scroll(modes[mode])
            show_time(get_clock_time())

    # detect a change in button B's state, the increment / select button
    button_b_state = button_b.is_pressed()
    if button_b_state != last_button_b_state:
        last_button_b_state = button_b_state

        if button_b_state == True:
            # button B's action depends on the current mode
            if mode == 0:   #show time
                display.scroll(decode_time(get_clock_time()))
            elif mode == 1: #setting time: increment hour units
                base_time = base_time - (60 * 60 * 1000)
            elif mode == 2: #setting time: increment minute tens
                base_time = base_time - (10 * 60 * 1000)
            elif mode == 3: #setting time: increment minute units
                base_time = base_time - (60 * 1000)

            show_time(get_clock_time())
  

    #If in clock mode update the display every second
    if mode == 0:
        display_time = running_time() - last_display_time
        if display_time >= 1000:
            last_display_time = display_time
            show_time(get_clock_time())

    sleep(100)



"""
Two-dimensional bubble level which uses the accelerometer.
"""

from microbit import *

sensitivity = 'medium'  # Change to 'low', 'medium', or 'high' to adjust
divisors = {'low':64, 'medium':32, 'high':16}

def clamp(number, min, max):
    """Returns number limited to range specified by min and max, inclusive"""
    if number < min:
        return min
    elif number > max:
        return max
    else:
        return number

while True:
    x_grav, y_grav, _ = accelerometer.get_values()
    # Map raw values from accelerometer to pixels on display
    x_pixel = 4 - clamp(2 + x_grav // divisors.get(sensitivity), 0, 4)
    y_pixel = 4 - clamp(2 + y_grav // divisors.get(sensitivity), 0, 4)
    display.clear()
    display.set_pixel(x_pixel, y_pixel, 9)
    sleep(100)